package RPM::Devscripts::Versort;

use strict;
use warnings;

use RPM::Vercmp;
use Exporter 'import';

our @EXPORT_OK = qw/upstream_versort/;
our $VERSION = 0.001;

sub trimmed_version {
    my $ver=shift;
    $ver=~s/(?: # select
	-(?:rel|release|current|pre\d{0,1}|dev|alpha\d{0,1}|beta\d{0,1})| # -pre2
	(?:rc[0-9]|[ab][0-9]) #a2 b3 rc4
	)$//x;
    return $ver;
}

sub trimmed_vercmp {
    my ($ver1, $ver2) = @_;
    return RPM::Vercmp::vercmp(&trimmed_version($ver1), '', &trimmed_version($ver2), '')<=>0;
}

sub upstream_versort (@)
{
    my @namever_pairs = @_;

    return sort { &trimmed_vercmp($$b[0], $$a[0]) } @namever_pairs;
}

1;

__END__

=head1	NAME

RPM::Devscripts::Versort - Debian Devscripts::Versort for rpm

=head1	DESCRIPTION

This module provides a part of Devscripts::Versort functionality
for the rpm ports of debian tools such as rpm-uscan.

=head1	STATIC FUNCTIONS

=over

=item sub upstream_versort (@)

upstream_versort take as input an array of elements of the form
[version, data, ...] and sort them into decreasing order according to rpm's
understanding of version sorting.  The output is a sorted array.

The returned array has the greatest version as the 0th array element.

=back

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2021 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

#
# It's based on of Devscript::Versort
#

# Copyright (C) 1998,2002 Julian Gilbey <jdg@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

# The functions in this Perl module are versort, upstream_versort and
# deb_versort.  They each take as input an array of elements of the form
# [version, data, ...] and sort them into decreasing order according to dpkg's
# understanding of version sorting.  The output is a sorted array.  In
# upstream_versort, "version" is assumed to be an upstream version number only,
# whereas in deb_versort, "version" is assumed to be a Debian version number,
# possibly including an epoch and/or a Debian revision. versort is available
# for compatibility reasons. It compares versions as Debian versions
# (i.e. 1-2-4 < 1-3) but disables checks for wellformed versions.
#
# The returned array has the greatest version as the 0th array element.

package Devscripts::Versort;
use Dpkg::Version;

sub versort (@) {
    return _versort(0, sub { return shift->[0] }, @_);
}

sub deb_versort (@) {
    return _versort(1, sub { return shift->[0] }, @_);
}

sub upstream_versort (@) {
    return _versort(0, sub { return "1:" . shift->[0] . "-0" }, @_);
}

sub _versort ($@) {
    my ($check, $getversion, @namever_pairs) = @_;

    foreach my $pair (@namever_pairs) {
        unshift(@$pair,
            Dpkg::Version->new(&$getversion($pair), check => $check));
    }

    my @sorted = sort { $b->[0] <=> $a->[0] } @namever_pairs;

    foreach my $pair (@sorted) {
        shift @$pair;
    }

    return @sorted;
}

1;
